package com.ruoyi.system.utils.dbsync.meta;

import com.alibaba.druid.util.JdbcUtils;
import com.ruoyi.system.utils.dbsync.util.SqlUtil;
import java.sql.Connection;
import java.sql.DriverManager;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.util.Iterator;
import java.util.LinkedHashMap;
import java.util.Map;

public class MetaData {
   private Connection conn;
   private String user;
   private String password;
   private String schema;
   private String jdbcUrl;
   private Map<String, Table> tables = new LinkedHashMap();

   public Boolean init() {
      try {
         Class.forName(JdbcUtils.getDriverClassName(this.jdbcUrl));
         this.conn = DriverManager.getConnection(this.jdbcUrl, this.user, this.password);
         this.getTablesFromDb();
         this.getCreateTables();
         this.getAllTableColumns();
         this.getAllTableKeys();
         return Boolean.TRUE;
      } catch (Exception var2) {
         return Boolean.FALSE;
      }
   }

   public Boolean connect() {
      try {
         Class.forName(JdbcUtils.getDriverClassName(this.jdbcUrl));
         this.conn = DriverManager.getConnection(this.jdbcUrl, this.user, this.password);
         return Boolean.TRUE;
      } catch (Exception var2) {
         return Boolean.FALSE;
      }
   }

   public void setAutoCommit(Boolean auto) throws SQLException {
      this.conn.setAutoCommit(auto);
   }

   public void Commit() throws SQLException {
      this.conn.commit();
   }

   private void getTablesFromDb() throws SQLException {
      ResultSet rs = SqlUtil.executeSql(this.conn, "show tables from `" + this.schema + "`");

      while(rs.next()) {
         Table table = new Table();
         table.setTableName(rs.getString("Tables_in_" + this.schema));
         this.tables.put(table.getTableName(), table);
      }

   }

   private void getCreateTables() throws SQLException {
      Iterator var1 = this.tables.values().iterator();

      while(var1.hasNext()) {
         Table table = (Table)var1.next();
         this.getCreateTable(table);
      }

   }

   private void getCreateTable(Table table) throws SQLException {
      ResultSet rs = SqlUtil.executeSql(this.conn, "show create table `" + this.schema + "`." + table.getTableName());

      while(rs.next()) {
         table.setCreateTable(rs.getString("Create Table"));
      }

   }

   private void getAllTableColumns() throws SQLException {
      Iterator var1 = this.tables.values().iterator();

      while(var1.hasNext()) {
         Table table = (Table)var1.next();
         this.getTableColumns(table);
      }

   }

   private void getTableColumns(Table table) throws SQLException {
      ResultSet rs = SqlUtil.executeSql(this.conn, "select COLUMN_NAME,COLUMN_TYPE,IS_NULLABLE,COLUMN_DEFAULT,COLUMN_COMMENT,EXTRA from information_schema.columns where TABLE_SCHEMA='" + this.schema + "' and TABLE_NAME='" + table.getTableName() + "' order by ORDINAL_POSITION asc");

      while(rs.next()) {
         Column column = new Column();
         column.setName(rs.getString("COLUMN_NAME"));
         column.setType(rs.getString("COLUMN_TYPE"));
         column.setDefaultValue(rs.getString("COLUMN_DEFAULT"));
         column.setIsNull(rs.getString("IS_NULLABLE"));
         column.setExtra(rs.getString("EXTRA"));
         column.setComment(rs.getString("COLUMN_COMMENT"));
         table.getColumns().put(column.getName(), column);
      }

   }

   private void getAllTableKeys() throws SQLException {
      Iterator var1 = this.tables.values().iterator();

      while(var1.hasNext()) {
         Table table = (Table)var1.next();
         this.getTableKeys(table);
      }

   }

   private void getTableKeys(Table table) throws SQLException {
      ResultSet rs = SqlUtil.executeSql(this.conn, "show keys from `" + this.schema + "`." + table.getTableName());
      Index last = null;
      Index tmp = null;

      while(true) {
         while(rs.next()) {
            String keyName = rs.getString("Key_name");
            if (last != null && keyName.equals(last.getIndexName())) {
               last.getColumns().add(rs.getString("Column_name"));
            } else {
               last = new Index();
               last.setIndexName(keyName);
               last.getColumns().add(rs.getString("Column_name"));
               last.setNotUnique(rs.getString("Non_unique"));
               table.getIndexes().put(last.getIndexName(), last);
            }
         }

         return;
      }
   }

   public String getJdbcUrl() {
      return this.jdbcUrl;
   }

   public void setJdbcUrl(String jdbcUrl) {
      this.jdbcUrl = jdbcUrl;
   }

   public String getSchema() {
      return this.schema;
   }

   public void setSchema(String schema) {
      this.schema = schema;
   }

   public String getUser() {
      return this.user;
   }

   public void setUser(String user) {
      this.user = user;
   }

   public String getPassword() {
      return this.password;
   }

   public void setPassword(String password) {
      this.password = password;
   }

   public Connection getConn() {
      return this.conn;
   }

   public void setConn(Connection conn) {
      this.conn = conn;
   }

   public Map<String, Table> getTables() {
      return this.tables;
   }

   public void setTables(Map<String, Table> tables) {
      this.tables = tables;
   }
}
